@@ -154,73 +154,11 @@ test_that("github regexes", {
   }
 })
 
-test_that("github url regexes", {
-  cases <- list(
-    list("https://github.com/u/repo.git", c(username = "u", repo = "repo")),
-    list("https://github.com/u/re.po", c(username = "u", repo = "re.po")),
-    list("https://github.com/u/re.po.git", c(username = "u", repo = "re.po"))
-  )
-  for (c in cases) {
-    m <- re_match(c[[1]], github_url_rx())
-    for (n in names(c[[2]])) expect_equal(c[[2]][[n]], m[[n]])
-  }
-})
-
-test_that("parse_pkg_refs, github", {
-
-  cases <- list(
-    list("user/repo"),
-    list("github::user/repo"),
-    list("pkg=user/repo", package = "pkg"),
-    list("pkg=github::user/repo", package = "pkg"),
-    list("user/repo/subdir", subdir = "subdir"),
-    list("user/repo@badcafe", commitish = "badcafe"),
-    list("user/repo#123", pull = "123"),
-    list("user/repo@*release", release = "*release"),
-    list("github::user/repo/subdir", subdir = "subdir"),
-    list("github::user/repo@badcafe", commitish = "badcafe"),
-    list("github::user/repo#123", pull = "123"),
-    list("github::user/repo@*release", release = "*release"),
-    list("pkg=user/repo/subdir", package = "pkg", subdir = "subdir"),
-    list("pkg=user/repo@badcafe", package = "pkg", commitish = "badcafe"),
-    list("pkg=user/repo#123", package = "pkg", pull = "123"),
-    list("pkg=user/repo@*release", package = "pkg", release = "*release"),
-
-    # github url cases
-    list("git@github.com:user/repo.git"),
-    list("git@github.ubc.ca:user/repo.git"),
-    list("https://github.com/user/repo"),
-    list("https://github.ubc.ca/user/repo"),
-    list("https://github.com/user/repo/tree/i-am-a-branch", commitish = "i-am-a-branch"),
-    list("https://github.com/user/repo/commit/1234567", commitish = "1234567"),
-    list("https://github.com/user/repo/pull/108", pull = "108"),
-    list("https://github.com/user/repo/releases/tag/1.0.0", commitish = "1.0.0"),
-    list("https://github.com/user/repo/releases/latest", release = "latest"),
-    list("https://github.com/user/repo/releases/latest", release = "latest"),
-    list("https://github.com/foo/bar", username = "foo", repo = "bar"),
-    list("git@github.com:foo/bar.git", username = "foo", repo = "bar"),
-
-    # Username and repo can have hyphens in them
-    list("git@github.com:foo-bar/baz-qux.git", username = "foo-bar", repo = "baz-qux")
-  )
-
-  for (case in cases) {
-    expect_equal_named_lists(
-      p <- parse_pkg_refs(case[[1]])[[1]],
-      utils::modifyList(
-        list(package = case$repo %||% "repo", username = "user",
-             repo = "repo", subdir = "", commitish = "", pull = "",
-             release = "", ref = case[[1]], type = "github",
-             params = character()),
-        case[-1]
-      )
-    )
-    expect_s3_class(p, c("remote_ref_github", "remote_ref"))
-  }
-})
-
 test_that("parse_pkg_refs error on unknown type", {
-  expect_error(parse_pkg_refs("my_package"), "parse remotes")
+  expect_snapshot(
+    error = TRUE,
+    parse_pkg_refs(c("notgood::pkg", "good", "my_package"))
+  )
 })
 
 test_that("custom remote types", {
@@ -302,7 +240,8 @@ test_that("parse_pkg_refs, local", {
     expect_equal(
       parse_pkg_ref(c[[1]]),
       structure(
-        list(path = c[[2]], ref = paste0("local::", c[[2]]), type = "local",
+        list(package = NA_character_, path = c[[2]],
+             ref = paste0("local::", c[[2]]), type = "local",
              params = character()),
         class = c("remote_ref_local", "remote_ref", "list")
       )
@@ -341,3 +280,20 @@ test_that("parameters", {
     expect_equal(wi, wo)
   }
 })
+
+test_that("explicit package names", {
+  expect_snapshot({
+    parse_pkg_ref("package=user/notpackage")
+    parse_pkg_ref("package=user/notpackage@tag")
+    parse_pkg_ref("package=github::user/notpackage@tag")
+
+    parse_pkg_ref("package=local::/abs/path")
+    parse_pkg_ref("package=local::rel/path")
+    parse_pkg_ref("package=local::~/home/path")
+    parse_pkg_ref("package=/abs/path")
+    parse_pkg_ref("package=./rel/path")
+    parse_pkg_ref("package=~/home/path")
+
+    parse_pkg_ref("package=url::https://example.com/p1.tar.gz")
+  })
+})
